IsoRes.Sprite = function(options){
	this.style = 'basic';
	
	/**
	 * The X coordinate of the tile on the screen at this time.
	 */
	this.x = 0;
	/**
	 * The y coordinate of the tile on the screen at this time.
	 */
	this.y = 0;
}

IsoRes.Sprite.prototype = {
	/**
	 * What is this we're dealing with?
	 */
	type : 'sprite',
	/**
	 * Has this sprite changed? If true, the sprite needs to be redrawn.
	 */
	changed:false,
	/** 
	 * The image or canvas element to draw.
	 */
	image : false,
	/**
	 * Direction in which a sprite is facing.
	 * Directions read clockwise from north to north-west. Halfway
	 * definitions are defined with the /first/ direction on the clock
	 * face defined first. For instance north-east becomes "ne", because
	 * north comes before east. South-east becomes "es" because east
	 * is before south.
	 * @see directions
	 */
	direction : 'n',
	/**
	 * The direction from which we are animating.
	 * Used if making the character perform an about face, for instance.
	 * prepareSprite will return each frame between these directions
	 * until the characer is facing the correct direction.
	 */
	directionFrom : false,
	/**
	 * A list of directions this sprite can face.
	 * These are synonymous with image files on the filesystem, or in
	 * the case of a sprite sheet, the index at which the specified
	 * sprite appears.
	 * These should be kept in clockwise order starting at north, as
	 * they define how a sprite "turns" or transitions between one
	 * direction and another.
	 * Enumerated to a key:pair object by enumerateDirections().
	 */
	directions : ['n','ne','e','es','s','sw','w','nw'],
	
	/**
	 * The animation frame this sprite is currently at. Animations
	 * are sprited in a downward direction.
	 */
	animationFrame : 0,
	
	/**
	 * The tile the sprite currently occupies
	 */
	locTile : [1,1],
	
	/**
	 * The subtile the sprite currently occupies. This is a 6*6 grid on
	 * a single tile. May be a decimal.
	 */
	locTileSub : [1,1],
	
	/**
	 * The width of a single sprite element.
	 */
	width:115,
	height:115,
	
	/**
	 * The offset to apply to position the sprite correctly on the tile.
	 * [x,y]
	 */
	spriteOffset : [0,0],
	
	/**
	 * The name of the sprite. Be sure to set this in your subclass,
	 * as all filename lookups refer to this.
	 */
	spriteName : false,
	
	/**
	 * Which renderer (IsoCan.Renderers) shouuld we use to display this?
	 */
	renderer : 'sprite',
	
	fps : false,
	
	animationFramesTotal : 0,
	
	highlightOnHover : true,
	
	save : function(){
		var savedSprite = _.extend({},this);
		_.each([
			'parent',
			'spriteSheet'
		],function(property){
			delete savedSprite[property];
		});
		return savedSprite;
	},
	loadSprite : function(){
		this.enumerateDirections();
		this.loadSpriteSheet();
	},
	prepareSprite : function(){
		this.sx = this.animationFrame*this.width;
		this.sy = this.height * this.directionsByIndex[this.direction];

		var subtilePx = this.parent.getLocSub(this.locTileSub[0],this.locTileSub[1]);
		this.dx = subtilePx[0]+this.spriteOffset[0];
		this.dy = subtilePx[1]+this.spriteOffset[1]-this.height;
	},
	drawSprite : function(time){
		// If the sprite has more than one frame, call the animator.
		if(this.animationFramesTotal){
			this.advanceFrame(time);
		}
		
		// If the sprite has changed, recalculate dimensions.
		if(this.changed){
			this.prepareSprite();
		}
		return this;
	},
	
	advanceFrame : function(time){
		
		this.animationFrame = Math.floor(time*this.fps%(this.animationFramesTotal+1));
		this.changed = true;
		
	},
	
	/**
	 * Enumerate sprite states (create a key:value of this.directions)
	 * Call this after you set this.directions on the fly.
	 */
	enumerateDirections : function(){
		this.directionsByIndex = {};
		for(direction in this.directions){
			this.directionsByIndex[this.directions[direction]] = Number(direction);
		};
	},
	loadSpriteSheet : function(){
		var parent = this;
		
		parent.spriteSheet = new Image();
		parent.spriteSheet.onload = function(){
			parent.changed = true;
			delete parent.spriteSheet.onload;
		}
		parent.spriteSheet.src = parent.parent.getImageUri(parent.spriteName,parent.style,'sprites');
	},
	/**
	 * Rotate the sprite one unit to the direction
	 * @param direction May be 'left' or 'right'
	 * @return this.setDirection
	 */
	turn : function(direction){
		var currentDirection = this.directionsByIndex[this.direction];
		var newDirection = currentDirection;
		if(direction == 'right'){
			if(++newDirection == this.directions.length){
				newDirection = 0;
			}
		} else {
			if(--newDirection < 0){
				newDirection = this.directions.length-1;
			}
		}
		return this.setDirection(this.directions[newDirection]);
	},
	/**
	 * Set the direction of the sprite.
	 * @param direction The string direction of the sprite.
	 * @see this.directions
	 */
	setDirection : function(direction){
		// Check if the direction is valid before applying it.
		if(isNaN(this.directionsByIndex[direction])){
			console.log('Direction ',direction,' is not a valid direction.');
		} else{
			// Set the directionFrom so we can animate between directions if applicable.
			if(!this.directionFrom){
				this.directionFrom = this.direction;
			}
			this.direction = direction;
			this.changed = true;
		}
		return this;
	},
	
	goTo : function(tile){
		this.locTile = [tile[0],tile[1]];
		this.locTileSub = [tile[3],tile[2]];
		this.changed = true;
	},
	
	/**
	 * Is the position within the bounds of the sprite? Used for
	 * collision detection & cursor detection.
	 * @param {array} bounds [x,y] object
	 * @returns {boolean} true on collision.
	 */
	 inBounds : function(bounds){
		 if(
			bounds[0] > this.x &&
			bounds[1] > this.y && 
			bounds[0] < this.x + this.width &&
			bounds[1] < this.y + this.height
		){
			return true;
		}
		return false;
	 }
	
}
